{% extends 'layout.html' %}
{% block content %}
<div class="page-header">
  <h1>Metric
    {% if name|length > 75 %}
    {% set name = "%s&hellip;"|format(name[:75])|safe %}
    {% else %}
    {% set name = name %}
    {% endif %}
    <small>{{name}}</small>
  </h1>
</div>
<table class="ui segment table">
  <thead>
    <tr>
      <th>Option</th>
      <th>Value</th>
    </tr>
  </thead>
  <tbody>
    {% for key,value in metric %}
    <tr>
      <td>{{key}}</td>
      {% if value is mapping %}
        <td><pre class="json">{{value|jsonprint}}</pre></td>
      {% else %}
        <td>{{value}}</td>
      {% endif %}
    </tr>
    {% endfor %}
  </tbody>
</table>
{% endblock content %}
