{% extends 'layout.html' %}
{% block content %}
<h2>Internal Server Error</h2>
<p>This error usually occurs because:
  <ul>
    <li>We were unable to reach PuppetDB;</li>
    <Li>The query to be executed was malformed resulting in an incorrectly encoded request.</li>
</ul></p>
<p>Please have a look at the log output for further information.</p>
{% endblock %}
